/*
 * DO NOT EDIT.  THIS FILE IS GENERATED FROM /builds/slave/mozilla-central-macosx-xulrunner/build/dom/indexedDB/nsIIDBIndexRequest.idl
 */

#ifndef __gen_nsIIDBIndexRequest_h__
#define __gen_nsIIDBIndexRequest_h__


#ifndef __gen_nsIIDBIndex_h__
#include "nsIIDBIndex.h"
#endif

/* For IDL files that don't want to include root IDL files. */
#ifndef NS_NO_VTABLE
#define NS_NO_VTABLE
#endif
class nsIIDBKeyRange; /* forward declaration */

class nsIIDBRequest; /* forward declaration */

class nsIVariant; /* forward declaration */


/* starting interface:    nsIIDBIndexRequest */
#define NS_IIDBINDEXREQUEST_IID_STR "8c21dfc3-62fa-470e-bc92-7d115b0fac97"

#define NS_IIDBINDEXREQUEST_IID \
  {0x8c21dfc3, 0x62fa, 0x470e, \
    { 0xbc, 0x92, 0x7d, 0x11, 0x5b, 0x0f, 0xac, 0x97 }}

/**
 * IDBIndexRequest interface.  See
 * http://dev.w3.org/2006/webapi/WebSimpleDB/#idl-def-IDBIndexRequest for more
 * information.
 */
class NS_NO_VTABLE NS_SCRIPTABLE nsIIDBIndexRequest : public nsIIDBIndex {
 public: 

  NS_DECLARE_STATIC_IID_ACCESSOR(NS_IIDBINDEXREQUEST_IID)

  /* nsIIDBRequest get (in nsIVariant key); */
  NS_SCRIPTABLE NS_IMETHOD Get(nsIVariant *key, nsIIDBRequest **_retval NS_OUTPARAM) = 0;

  /* nsIIDBRequest getObject (in nsIVariant key); */
  NS_SCRIPTABLE NS_IMETHOD GetObject(nsIVariant *key, nsIIDBRequest **_retval NS_OUTPARAM) = 0;

  /* [optional_argc] nsIIDBRequest getAll ([optional] in nsIVariant key, [optional] in unsigned long limit); */
  NS_SCRIPTABLE NS_IMETHOD GetAll(nsIVariant *key, PRUint32 limit, PRUint8 _argc, nsIIDBRequest **_retval NS_OUTPARAM) = 0;

  /* [optional_argc] nsIIDBRequest getAllObjects ([optional] in nsIVariant key, [optional] in unsigned long limit); */
  NS_SCRIPTABLE NS_IMETHOD GetAllObjects(nsIVariant *key, PRUint32 limit, PRUint8 _argc, nsIIDBRequest **_retval NS_OUTPARAM) = 0;

  /* [optional_argc] nsIIDBRequest openCursor ([optional] in nsIIDBKeyRange range, [optional] in unsigned short direction, [optional] in boolean preload); */
  NS_SCRIPTABLE NS_IMETHOD OpenCursor(nsIIDBKeyRange *range, PRUint16 direction, PRBool preload, PRUint8 _argc, nsIIDBRequest **_retval NS_OUTPARAM) = 0;

  /* [optional_argc] nsIIDBRequest openObjectCursor ([optional] in nsIIDBKeyRange range, [optional] in unsigned short direction, [optional] in boolean preload); */
  NS_SCRIPTABLE NS_IMETHOD OpenObjectCursor(nsIIDBKeyRange *range, PRUint16 direction, PRBool preload, PRUint8 _argc, nsIIDBRequest **_retval NS_OUTPARAM) = 0;

};

  NS_DEFINE_STATIC_IID_ACCESSOR(nsIIDBIndexRequest, NS_IIDBINDEXREQUEST_IID)

/* Use this macro when declaring classes that implement this interface. */
#define NS_DECL_NSIIDBINDEXREQUEST \
  NS_SCRIPTABLE NS_IMETHOD Get(nsIVariant *key, nsIIDBRequest **_retval NS_OUTPARAM); \
  NS_SCRIPTABLE NS_IMETHOD GetObject(nsIVariant *key, nsIIDBRequest **_retval NS_OUTPARAM); \
  NS_SCRIPTABLE NS_IMETHOD GetAll(nsIVariant *key, PRUint32 limit, PRUint8 _argc, nsIIDBRequest **_retval NS_OUTPARAM); \
  NS_SCRIPTABLE NS_IMETHOD GetAllObjects(nsIVariant *key, PRUint32 limit, PRUint8 _argc, nsIIDBRequest **_retval NS_OUTPARAM); \
  NS_SCRIPTABLE NS_IMETHOD OpenCursor(nsIIDBKeyRange *range, PRUint16 direction, PRBool preload, PRUint8 _argc, nsIIDBRequest **_retval NS_OUTPARAM); \
  NS_SCRIPTABLE NS_IMETHOD OpenObjectCursor(nsIIDBKeyRange *range, PRUint16 direction, PRBool preload, PRUint8 _argc, nsIIDBRequest **_retval NS_OUTPARAM); 

/* Use this macro to declare functions that forward the behavior of this interface to another object. */
#define NS_FORWARD_NSIIDBINDEXREQUEST(_to) \
  NS_SCRIPTABLE NS_IMETHOD Get(nsIVariant *key, nsIIDBRequest **_retval NS_OUTPARAM) { return _to Get(key, _retval); } \
  NS_SCRIPTABLE NS_IMETHOD GetObject(nsIVariant *key, nsIIDBRequest **_retval NS_OUTPARAM) { return _to GetObject(key, _retval); } \
  NS_SCRIPTABLE NS_IMETHOD GetAll(nsIVariant *key, PRUint32 limit, PRUint8 _argc, nsIIDBRequest **_retval NS_OUTPARAM) { return _to GetAll(key, limit, _argc, _retval); } \
  NS_SCRIPTABLE NS_IMETHOD GetAllObjects(nsIVariant *key, PRUint32 limit, PRUint8 _argc, nsIIDBRequest **_retval NS_OUTPARAM) { return _to GetAllObjects(key, limit, _argc, _retval); } \
  NS_SCRIPTABLE NS_IMETHOD OpenCursor(nsIIDBKeyRange *range, PRUint16 direction, PRBool preload, PRUint8 _argc, nsIIDBRequest **_retval NS_OUTPARAM) { return _to OpenCursor(range, direction, preload, _argc, _retval); } \
  NS_SCRIPTABLE NS_IMETHOD OpenObjectCursor(nsIIDBKeyRange *range, PRUint16 direction, PRBool preload, PRUint8 _argc, nsIIDBRequest **_retval NS_OUTPARAM) { return _to OpenObjectCursor(range, direction, preload, _argc, _retval); } 

/* Use this macro to declare functions that forward the behavior of this interface to another object in a safe way. */
#define NS_FORWARD_SAFE_NSIIDBINDEXREQUEST(_to) \
  NS_SCRIPTABLE NS_IMETHOD Get(nsIVariant *key, nsIIDBRequest **_retval NS_OUTPARAM) { return !_to ? NS_ERROR_NULL_POINTER : _to->Get(key, _retval); } \
  NS_SCRIPTABLE NS_IMETHOD GetObject(nsIVariant *key, nsIIDBRequest **_retval NS_OUTPARAM) { return !_to ? NS_ERROR_NULL_POINTER : _to->GetObject(key, _retval); } \
  NS_SCRIPTABLE NS_IMETHOD GetAll(nsIVariant *key, PRUint32 limit, PRUint8 _argc, nsIIDBRequest **_retval NS_OUTPARAM) { return !_to ? NS_ERROR_NULL_POINTER : _to->GetAll(key, limit, _argc, _retval); } \
  NS_SCRIPTABLE NS_IMETHOD GetAllObjects(nsIVariant *key, PRUint32 limit, PRUint8 _argc, nsIIDBRequest **_retval NS_OUTPARAM) { return !_to ? NS_ERROR_NULL_POINTER : _to->GetAllObjects(key, limit, _argc, _retval); } \
  NS_SCRIPTABLE NS_IMETHOD OpenCursor(nsIIDBKeyRange *range, PRUint16 direction, PRBool preload, PRUint8 _argc, nsIIDBRequest **_retval NS_OUTPARAM) { return !_to ? NS_ERROR_NULL_POINTER : _to->OpenCursor(range, direction, preload, _argc, _retval); } \
  NS_SCRIPTABLE NS_IMETHOD OpenObjectCursor(nsIIDBKeyRange *range, PRUint16 direction, PRBool preload, PRUint8 _argc, nsIIDBRequest **_retval NS_OUTPARAM) { return !_to ? NS_ERROR_NULL_POINTER : _to->OpenObjectCursor(range, direction, preload, _argc, _retval); } 

#if 0
/* Use the code below as a template for the implementation class for this interface. */

/* Header file */
class nsIDBIndexRequest : public nsIIDBIndexRequest
{
public:
  NS_DECL_ISUPPORTS
  NS_DECL_NSIIDBINDEXREQUEST

  nsIDBIndexRequest();

private:
  ~nsIDBIndexRequest();

protected:
  /* additional members */
};

/* Implementation file */
NS_IMPL_ISUPPORTS1(nsIDBIndexRequest, nsIIDBIndexRequest)

nsIDBIndexRequest::nsIDBIndexRequest()
{
  /* member initializers and constructor code */
}

nsIDBIndexRequest::~nsIDBIndexRequest()
{
  /* destructor code */
}

/* nsIIDBRequest get (in nsIVariant key); */
NS_IMETHODIMP nsIDBIndexRequest::Get(nsIVariant *key, nsIIDBRequest **_retval NS_OUTPARAM)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* nsIIDBRequest getObject (in nsIVariant key); */
NS_IMETHODIMP nsIDBIndexRequest::GetObject(nsIVariant *key, nsIIDBRequest **_retval NS_OUTPARAM)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* [optional_argc] nsIIDBRequest getAll ([optional] in nsIVariant key, [optional] in unsigned long limit); */
NS_IMETHODIMP nsIDBIndexRequest::GetAll(nsIVariant *key, PRUint32 limit, PRUint8 _argc, nsIIDBRequest **_retval NS_OUTPARAM)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* [optional_argc] nsIIDBRequest getAllObjects ([optional] in nsIVariant key, [optional] in unsigned long limit); */
NS_IMETHODIMP nsIDBIndexRequest::GetAllObjects(nsIVariant *key, PRUint32 limit, PRUint8 _argc, nsIIDBRequest **_retval NS_OUTPARAM)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* [optional_argc] nsIIDBRequest openCursor ([optional] in nsIIDBKeyRange range, [optional] in unsigned short direction, [optional] in boolean preload); */
NS_IMETHODIMP nsIDBIndexRequest::OpenCursor(nsIIDBKeyRange *range, PRUint16 direction, PRBool preload, PRUint8 _argc, nsIIDBRequest **_retval NS_OUTPARAM)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* [optional_argc] nsIIDBRequest openObjectCursor ([optional] in nsIIDBKeyRange range, [optional] in unsigned short direction, [optional] in boolean preload); */
NS_IMETHODIMP nsIDBIndexRequest::OpenObjectCursor(nsIIDBKeyRange *range, PRUint16 direction, PRBool preload, PRUint8 _argc, nsIIDBRequest **_retval NS_OUTPARAM)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* End of implementation class template. */
#endif


#endif /* __gen_nsIIDBIndexRequest_h__ */
